@@ -27,7 +27,13 @@
        used to specify information about the request.
 """
 
-__author__ = 'api.jscudder (Jeffrey Scudder)'
+__author__ = 'api.jscudder (Jeff Scudder)'
+
+
+import atom.http_interface
+import atom.url
+import atom.http
+import atom.token_store
 
 import os
 import httplib
@@ -35,6 +41,7 @@ import urllib
 import re
 import base64
 import socket
+import warnings
 try:
   from xml.etree import cElementTree as ElementTree
 except ImportError:
@@ -47,9 +54,6 @@ except ImportError:
       from elementtree import ElementTree
 
 
-URL_REGEX = re.compile('http(s)?\://([\w\.-]*)(\:(\d+))?(/.*)?')
-
-
 class AtomService(object):
   """Performs Atom Publishing Protocol CRUD operations.
   
@@ -59,10 +63,22 @@ class AtomService(object):
   # Default values for members
   port = 80
   ssl = False
-  # If debug is True, the HTTPConnection will display debug information
-  debug = False
+  # Set the current_token to force the AtomService to use this token
+  # instead of searching for an appropriate token in the token_store.
+  current_token = None
+  auto_store_tokens = True
+  auto_set_current_token = True
 
-  def __init__(self, server=None, additional_headers=None):
+  def _get_override_token(self):
+    return self.current_token
+
+  def _set_override_token(self, token):
+    self.current_token = token
+
+  override_token = property(_get_override_token, _set_override_token)
+
+  def __init__(self, server=None, additional_headers=None, 
+      application_name='', http_client=None, token_store=None):
     """Creates a new AtomService client.
     
     Args:
@@ -71,20 +87,50 @@ class AtomService(object):
               'www.google.com'
       additional_headers: dict (optional) Any additional HTTP headers which
                           should be included with CRUD operations.
+      http_client: An object responsible for making HTTP requests using a
+                   request method. If none is provided, a new instance of
+                   atom.http.ProxiedHttpClient will be used.
+      token_store: Keeps a collection of authorization tokens which can be
+                   applied to requests for a specific URLs. Critical methods are
+                   find_token based on a URL (atom.url.Url or a string), add_token,
+                   and remove_token.
     """
-
+    self.http_client = http_client or atom.http.ProxiedHttpClient()
+    self.token_store = token_store or atom.token_store.TokenStore()
     self.server = server
     self.additional_headers = additional_headers or {}
+    self.additional_headers['User-Agent'] = atom.http_interface.USER_AGENT % (
+        application_name,)
+    # If debug is True, the HTTPConnection will display debug information
+    self._set_debug(False)
 
-    self.additional_headers['User-Agent'] = 'Python Google Data Client Lib'
+  def _get_debug(self):
+    return self.http_client.debug
 
-  def _ProcessUrl(self, url, for_proxy=False):
-    """Processes a passed URL.  If the URL does not begin with https?, then
-    the default value for self.server is used"""
-    return ProcessUrl(self, url, for_proxy=for_proxy)
+  def _set_debug(self, value):
+    self.http_client.debug = value
+
+  debug = property(_get_debug, _set_debug, 
+      doc='If True, HTTP debug information is printed.')
+
+  def use_basic_auth(self, username, password, scopes=None):
+    if username is not None and password is not None:
+      if scopes is None:
+        scopes = [atom.token_store.SCOPE_ALL]
+      base_64_string = base64.encodestring('%s:%s' % (username, password))
+      token = BasicAuthToken('Basic %s' % base_64_string.strip(), 
+          scopes=[atom.token_store.SCOPE_ALL])
+      if self.auto_set_current_token:
+        self.current_token = token
+      if self.auto_store_tokens:
+        return self.token_store.add_token(token)
+      return True
+    return False
 
   def UseBasicAuth(self, username, password, for_proxy=False):
     """Sets an Authenticaiton: Basic HTTP header containing plaintext.
+
+    Deprecated, use use_basic_auth instead.
     
     The username and password are base64 encoded and added to an HTTP header
     which will be included in each request. Note that your username and 
@@ -94,31 +140,41 @@ class AtomService(object):
       username: str
       password: str
     """
-    UseBasicAuth(self, username, password, for_proxy=for_proxy)
+    self.use_basic_auth(username, password)
 
-  def PrepareConnection(self, full_uri):
-    """Opens a connection to the server based on the full URI.
+  def request(self, operation, url, data=None, headers=None, 
+      url_params=None):
+    if isinstance(url, str):
+      if not url.startswith('http') and self.ssl:
+        url = atom.url.parse_url('https://%s%s' % (self.server, url))
+      elif not url.startswith('http'):
+        url = atom.url.parse_url('http://%s%s' % (self.server, url))
+      else:
+        url = atom.url.parse_url(url)
 
-    Examines the target URI and the proxy settings, which are set as 
-    environment variables, to open a connection with the server. This 
-    connection is used to make an HTTP request.
+    if url_params:
+      for name, value in url_params.iteritems():
+        url.params[name] = value
 
-    Args:
-      full_uri: str Which is the target relative (lacks protocol and host) or
-      absolute URL to be opened. Example:
-      'https://www.google.com/accounts/ClientLogin' or
-      'base/feeds/snippets' where the server is set to www.google.com.
+    all_headers = self.additional_headers.copy()
+    if headers:
+      all_headers.update(headers)
 
-    Returns:
-      A tuple containing the httplib.HTTPConnection and the full_uri for the
-      request.
-    """
-    return PrepareConnection(self, full_uri)   
+    # If the list of headers does not include a Content-Length, attempt to
+    # calculate it based on the data object.
+    if data and 'Content-Length' not in all_headers:
+      content_length = CalculateDataLength(data)
+      if content_length:
+        all_headers['Content-Length'] = str(content_length)
+
+    # Find an Authorization token for this URL if one is available.
+    if self.override_token:
+      auth_token = self.override_token
+    else:
+      auth_token = self.token_store.find_token(url)
+    return auth_token.perform_request(self.http_client, operation, url, 
+        data=data, headers=all_headers)
 
-  # Alias the old name for the above method to preserve backwards 
-  # compatibility.
-  _PrepareConnection = PrepareConnection
- 
   # CRUD operations
   def Get(self, uri, extra_headers=None, url_params=None, escape_params=True):
     """Query the APP server with the given URI
@@ -153,8 +209,8 @@ class AtomService(object):
     Returns:
       httplib.HTTPResponse The server's response to the GET request.
     """
-    return HttpRequest(self, 'GET', None, uri, extra_headers=extra_headers, 
-        url_params=url_params, escape_params=escape_params)
+    return self.request('GET', uri, data=None, headers=extra_headers, 
+                        url_params=url_params)
 
   def Post(self, data, uri, extra_headers=None, url_params=None, 
            escape_params=True, content_type='application/atom+xml'):
@@ -181,9 +237,12 @@ class AtomService(object):
     Returns:
       httplib.HTTPResponse Server's response to the POST request.
     """
-    return HttpRequest(self, 'POST', data, uri, extra_headers=extra_headers, 
-        url_params=url_params, escape_params=escape_params, 
-        content_type=content_type)
+    if extra_headers is None:
+      extra_headers = {}
+    if content_type:
+      extra_headers['Content-Type'] = content_type
+    return self.request('POST', uri, data=data, headers=extra_headers, 
+                        url_params=url_params)
 
   def Put(self, data, uri, extra_headers=None, url_params=None, 
            escape_params=True, content_type='application/atom+xml'):
@@ -210,9 +269,12 @@ class AtomService(object):
     Returns:
       httplib.HTTPResponse Server's response to the PUT request.
     """
-    return HttpRequest(self, 'PUT', data, uri, extra_headers=extra_headers, 
-        url_params=url_params, escape_params=escape_params, 
-        content_type=content_type)
+    if extra_headers is None:
+      extra_headers = {}
+    if content_type:
+      extra_headers['Content-Type'] = content_type
+    return self.request('PUT', uri, data=data, headers=extra_headers, 
+                        url_params=url_params)
 
   def Delete(self, uri, extra_headers=None, url_params=None, 
              escape_params=True):
@@ -237,145 +299,64 @@ class AtomService(object):
     Returns:
       httplib.HTTPResponse Server's response to the DELETE request.
     """
-    return HttpRequest(self, 'DELETE', None, uri, extra_headers=extra_headers, 
-        url_params=url_params, escape_params=escape_params)
+    return self.request('DELETE', uri, data=None, headers=extra_headers, 
+                        url_params=url_params)
 
 
-def HttpRequest(service, operation, data, uri, extra_headers=None, 
-    url_params=None, escape_params=True, content_type='application/atom+xml'):
-  """Performs an HTTP call to the server, supports GET, POST, PUT, and DELETE.
+class BasicAuthToken(atom.http_interface.GenericToken):
+  def __init__(self, auth_header, scopes=None):
+    """Creates a token used to add Basic Auth headers to HTTP requests.
 
-  Usage example, perform and HTTP GET on http://www.google.com/:
-    import atom.service
-    client = atom.service.AtomService()
-    http_response = client.Get('http://www.google.com/')
-  or you could set the client.server to 'www.google.com' and use the 
-  following:
-    client.server = 'www.google.com'
-    http_response = client.Get('/')
+    Args:
+      auth_header: str The value for the Authorization header.
+      scopes: list of str or atom.url.Url specifying the beginnings of URLs
+          for which this token can be used. For example, if scopes contains
+          'http://example.com/foo', then this token can be used for a request to
+          'http://example.com/foo/bar' but it cannot be used for a request to
+          'http://example.com/baz'
+    """
+    self.auth_header = auth_header
+    self.scopes = scopes or []
 
-  Args:
-    service: atom.AtomService object which contains some of the parameters 
-        needed to make the request. The following members are used to 
-        construct the HTTP call: server (str), additional_headers (dict), 
-        port (int), and ssl (bool).
-    operation: str The HTTP operation to be performed. This is usually one of
-        'GET', 'POST', 'PUT', or 'DELETE'
-    data: ElementTree, filestream, list of parts, or other object which can be 
-        converted to a string. 
-        Should be set to None when performing a GET or PUT.
-        If data is a file-like object which can be read, this method will read
-        a chunk of 100K bytes at a time and send them. 
-        If the data is a list of parts to be sent, each part will be evaluated
-        and sent.
-    uri: The beginning of the URL to which the request should be sent. 
-        Examples: '/', '/base/feeds/snippets', 
-        '/m8/feeds/contacts/default/base'
-    extra_headers: dict of strings. HTTP headers which should be sent
-        in the request. These headers are in addition to those stored in 
-        service.additional_headers.
-    url_params: dict of strings. Key value pairs to be added to the URL as
-        URL parameters. For example {'foo':'bar', 'test':'param'} will 
-        become ?foo=bar&test=param.
-    escape_params: bool default True. If true, the keys and values in 
-        url_params will be URL escaped when the form is constructed 
-        (Special characters converted to %XX form.)
-    content_type: str The MIME type for the data being sent. Defaults to
-        'application/atom+xml', this is only used if data is set.
-  """
-  full_uri = BuildUri(uri, url_params, escape_params)
-  (connection, full_uri) = PrepareConnection(service, full_uri)
-
-  if extra_headers is None:
-    extra_headers = {}
-
-  # Turn on debug mode if the debug member is set.
-  if service.debug:
-    connection.debuglevel = 1
-
-  connection.putrequest(operation, full_uri)
-
-  # If the list of headers does not include a Content-Length, attempt to 
-  # calculate it based on the data object.
-  if (data and not service.additional_headers.has_key('Content-Length') and 
-      not extra_headers.has_key('Content-Length')):
-    content_length = __CalculateDataLength(data)
-    if content_length:
-      extra_headers['Content-Length'] = content_length
-
-  if content_type:
-    extra_headers['Content-Type'] = content_type 
-
-  # Send the HTTP headers.
-  if isinstance(service.additional_headers, dict):
-    for header in service.additional_headers:
-      connection.putheader(header, service.additional_headers[header])
-  if isinstance(extra_headers, dict):
-    for header in extra_headers:
-      connection.putheader(header, extra_headers[header])
-  connection.endheaders()
-
-  # If there is data, send it in the request.
-  if data:
-    if isinstance(data, list):
-      for data_part in data:
-        __SendDataPart(data_part, connection)
+  def perform_request(self, http_client, operation, url, data=None,
+                      headers=None):
+    """Sets the Authorization header to the basic auth string."""
+    if headers is None:
+      headers = {'Authorization':self.auth_header}
     else:
-      __SendDataPart(data, connection)
+      headers['Authorization'] = self.auth_header
+    return http_client.request(operation, url, data=data, headers=headers)
 
-  # Return the HTTP Response from the server.
-  return connection.getresponse()
+  def __str__(self):
+    return self.auth_header
 
-
-def __SendDataPart(data, connection):
-  if isinstance(data, str):
-    #TODO add handling for unicode.
-    connection.send(data)
-    return
-  elif ElementTree.iselement(data):
-    connection.send(ElementTree.tostring(data))
-    return
-  # Check to see if data is a file-like object that has a read method.
-  elif hasattr(data, 'read'):
-    # Read the file and send it a chunk at a time.
-    while 1:
-      binarydata = data.read(100000)
-      if binarydata == '': break
-      connection.send(binarydata)
-    return
-  else:
-    # The data object was not a file.
-    # Try to convert to a string and send the data.
-    connection.send(str(data))
-    return
-
-
-def __CalculateDataLength(data):
-  """Attempts to determine the length of the data to send. 
-  
-  This method will respond with a length only if the data is a string or
-  and ElementTree element.
-
-  Args:
-    data: object If this is not a string or ElementTree element this funtion
-        will return None.
-  """
-  if isinstance(data, str):
-    return len(data)
-  elif isinstance(data, list):
-    return None
-  elif ElementTree.iselement(data):
-    return len(ElementTree.tostring(data))
-  elif hasattr(data, 'read'):
-    # If this is a file-like object, don't try to guess the length.
-    return None
-  else:
-    return len(str(data))
+  def valid_for_scope(self, url):
+    """Tells the caller if the token authorizes access to the desired URL.
+    """
+    if isinstance(url, (str, unicode)):
+      url = atom.url.parse_url(url)
+    for scope in self.scopes:
+      if scope == atom.token_store.SCOPE_ALL:
+        return True
+      if isinstance(scope, (str, unicode)):
+        scope = atom.url.parse_url(scope)
+      if scope == url:
+        return True
+      # Check the host and the path, but ignore the port and protocol.
+      elif scope.host == url.host and not scope.path:
+        return True
+      elif scope.host == url.host and scope.path and not url.path:
+        continue
+      elif scope.host == url.host and url.path.startswith(scope.path):
+        return True
+    return False
 
 
 def PrepareConnection(service, full_uri):
   """Opens a connection to the server based on the full URI.
 
+  This method is deprecated, instead use atom.http.HttpClient.request.
+
   Examines the target URI and the proxy settings, which are set as
   environment variables, to open a connection with the server. This
   connection is used to make an HTTP request.
@@ -393,7 +374,7 @@ def PrepareConnection(service, full_uri):
     A tuple containing the httplib.HTTPConnection and the full_uri for the
     request.
   """
-   
+  deprecation('calling deprecated function PrepareConnection')
   (server, port, ssl, partial_uri) = ProcessUrl(service, full_uri)
   if ssl:
     # destination is https
@@ -474,6 +455,8 @@ def PrepareConnection(service, full_uri):
 
 def UseBasicAuth(service, username, password, for_proxy=False):
   """Sets an Authenticaiton: Basic HTTP header containing plaintext.
+
+  Deprecated, use AtomService.use_basic_auth insread.
   
   The username and password are base64 encoded and added to an HTTP header
   which will be included in each request. Note that your username and 
@@ -486,6 +469,7 @@ def UseBasicAuth(service, username, password, for_proxy=False):
     username: str
     password: str
   """
+  deprecation('calling deprecated function UseBasicAuth')
   base_64_string = base64.encodestring('%s:%s' % (username, password))
   base_64_string = base_64_string.strip()
   if for_proxy:
@@ -497,45 +481,43 @@ def UseBasicAuth(service, username, password, for_proxy=False):
 
 def ProcessUrl(service, url, for_proxy=False):
   """Processes a passed URL.  If the URL does not begin with https?, then
-  the default value for server is used"""
+  the default value for server is used
 
-  server = None
-  port = 80
+  This method is deprecated, use atom.url.parse_url instead.
+  """
+  if not isinstance(url, atom.url.Url):
+    url = atom.url.parse_url(url)
+
+  server = url.host
   ssl = False
-  if hasattr(service, 'server'):
-    server = service.server
-  else:
-    server = service
-  if not for_proxy:
+  port = 80
+
+  if not server:
+    if hasattr(service, 'server'):
+      server = service.server
+    else:
+      server = service
+    if not url.protocol and hasattr(service, 'ssl'):
+      ssl = service.ssl
     if hasattr(service, 'port'):
       port = service.port
-    if hasattr(service, 'ssl'):
-      ssl = service.ssl
-  uri = url
-
-  m = URL_REGEX.match(url)
-
-  if m is None:
-    return (server, port, ssl, uri)
   else:
-    if m.group(1) is not None:
-      port = 443
+    if url.protocol == 'https':
       ssl = True
-    if m.group(3) is None:
-      server = m.group(2)
-    else:
-      server = m.group(2)
-      port = int(m.group(4))
-    if m.group(5) is not None:
-      uri = m.group(5)
-    else:
-      uri = '/'
-    return (server, port, ssl, uri)
+    elif url.protocol == 'http':
+      ssl = False
+    if url.port:
+      port = int(url.port)
+    elif port == 80 and ssl:
+      port = 443
 
+  return (server, port, ssl, url.get_request_uri())
 
 def DictionaryToParamList(url_parameters, escape_params=True):
   """Convert a dictionary of URL arguments into a URL parameter string.
 
+  This function is deprcated, use atom.url.Url instead.
+
   Args:
     url_parameters: The dictionaty of key-value pairs which will be converted
                     into URL parameters. For example,
@@ -555,13 +537,14 @@ def DictionaryToParamList(url_parameters, escape_params=True):
                      for param, value in (url_parameters or {}).items()]
   # Turn parameter-value tuples into a list of strings in the form
   # 'PARAMETER=VALUE'.
-
   return ['='.join(x) for x in parameter_tuples]
 
 
 def BuildUri(uri, url_params=None, escape_params=True):
   """Converts a uri string and a collection of parameters into a URI.
 
+  This function is deprcated, use atom.url.Url instead.
+
   Args:
     uri: string
     url_params: dict (optional)
@@ -600,3 +583,144 @@ def BuildUri(uri, url_params=None, escape_params=True):
     full_uri = uri
         
   return full_uri
+
+  
+def HttpRequest(service, operation, data, uri, extra_headers=None, 
+    url_params=None, escape_params=True, content_type='application/atom+xml'):
+  """Performs an HTTP call to the server, supports GET, POST, PUT, and DELETE.
+  
+  This method is deprecated, use atom.http.HttpClient.request instead.
+
+  Usage example, perform and HTTP GET on http://www.google.com/:
+    import atom.service
+    client = atom.service.AtomService()
+    http_response = client.Get('http://www.google.com/')
+  or you could set the client.server to 'www.google.com' and use the 
+  following:
+    client.server = 'www.google.com'
+    http_response = client.Get('/')
+
+  Args:
+    service: atom.AtomService object which contains some of the parameters 
+        needed to make the request. The following members are used to 
+        construct the HTTP call: server (str), additional_headers (dict), 
+        port (int), and ssl (bool).
+    operation: str The HTTP operation to be performed. This is usually one of
+        'GET', 'POST', 'PUT', or 'DELETE'
+    data: ElementTree, filestream, list of parts, or other object which can be 
+        converted to a string. 
+        Should be set to None when performing a GET or PUT.
+        If data is a file-like object which can be read, this method will read
+        a chunk of 100K bytes at a time and send them. 
+        If the data is a list of parts to be sent, each part will be evaluated
+        and sent.
+    uri: The beginning of the URL to which the request should be sent. 
+        Examples: '/', '/base/feeds/snippets', 
+        '/m8/feeds/contacts/default/base'
+    extra_headers: dict of strings. HTTP headers which should be sent
+        in the request. These headers are in addition to those stored in 
+        service.additional_headers.
+    url_params: dict of strings. Key value pairs to be added to the URL as
+        URL parameters. For example {'foo':'bar', 'test':'param'} will 
+        become ?foo=bar&test=param.
+    escape_params: bool default True. If true, the keys and values in 
+        url_params will be URL escaped when the form is constructed 
+        (Special characters converted to %XX form.)
+    content_type: str The MIME type for the data being sent. Defaults to
+        'application/atom+xml', this is only used if data is set.
+  """
+  deprecation('call to deprecated function HttpRequest')
+  full_uri = BuildUri(uri, url_params, escape_params)
+  (connection, full_uri) = PrepareConnection(service, full_uri)
+
+  if extra_headers is None:
+    extra_headers = {}
+
+  # Turn on debug mode if the debug member is set.
+  if service.debug:
+    connection.debuglevel = 1
+
+  connection.putrequest(operation, full_uri)
+
+  # If the list of headers does not include a Content-Length, attempt to 
+  # calculate it based on the data object.
+  if (data and not service.additional_headers.has_key('Content-Length') and 
+      not extra_headers.has_key('Content-Length')):
+    content_length = CalculateDataLength(data)
+    if content_length:
+      extra_headers['Content-Length'] = str(content_length)
+
+  if content_type:
+    extra_headers['Content-Type'] = content_type 
+
+  # Send the HTTP headers.
+  if isinstance(service.additional_headers, dict):
+    for header in service.additional_headers:
+      connection.putheader(header, service.additional_headers[header])
+  if isinstance(extra_headers, dict):
+    for header in extra_headers:
+      connection.putheader(header, extra_headers[header])
+  connection.endheaders()
+
+  # If there is data, send it in the request.
+  if data:
+    if isinstance(data, list):
+      for data_part in data:
+        __SendDataPart(data_part, connection)
+    else:
+      __SendDataPart(data, connection)
+
+  # Return the HTTP Response from the server.
+  return connection.getresponse()
+  
+
+def __SendDataPart(data, connection):
+  """This method is deprecated, use atom.http._send_data_part"""
+  deprecated('call to deprecated function __SendDataPart')
+  if isinstance(data, str):
+    #TODO add handling for unicode.
+    connection.send(data)
+    return
+  elif ElementTree.iselement(data):
+    connection.send(ElementTree.tostring(data))
+    return
+  # Check to see if data is a file-like object that has a read method.
+  elif hasattr(data, 'read'):
+    # Read the file and send it a chunk at a time.
+    while 1:
+      binarydata = data.read(100000)
+      if binarydata == '': break
+      connection.send(binarydata)
+    return
+  else:
+    # The data object was not a file.
+    # Try to convert to a string and send the data.
+    connection.send(str(data))
+    return
+
+
+def CalculateDataLength(data):
+  """Attempts to determine the length of the data to send. 
+  
+  This method will respond with a length only if the data is a string or
+  and ElementTree element.
+
+  Args:
+    data: object If this is not a string or ElementTree element this funtion
+        will return None.
+  """
+  if isinstance(data, str):
+    return len(data)
+  elif isinstance(data, list):
+    return None
+  elif ElementTree.iselement(data):
+    return len(ElementTree.tostring(data))
+  elif hasattr(data, 'read'):
+    # If this is a file-like object, don't try to guess the length.
+    return None
+  else:
+    return len(str(data))
+    
+
+def deprecation(message):
+  warnings.warn(message, DeprecationWarning, stacklevel=2)
